import { toText } from 'myst-common';
import { xml2js } from 'xml-js';
import doi from 'doi-utils';
import { authorAndAffiliation, convertToUnist, findArticleId, toDate } from './utils';
import { select as unistSelect, selectAll } from 'unist-util-select';
import { Tags } from './types';
import { tic } from 'myst-cli-utils';
function select(selector, node) {
    var _a;
    return ((_a = unistSelect(selector, node)) !== null && _a !== void 0 ? _a : undefined);
}
export class Jats {
    constructor(data, opts) {
        var _a;
        const toc = tic();
        this.log = opts === null || opts === void 0 ? void 0 : opts.log;
        if (opts === null || opts === void 0 ? void 0 : opts.source)
            this.source = opts.source;
        try {
            this.raw = xml2js(data, { compact: false });
        }
        catch (error) {
            throw new Error('Problem parsing the JATS document, please ensure it is XML');
        }
        const { declaration, elements } = this.raw;
        this.declaration = declaration === null || declaration === void 0 ? void 0 : declaration.attributes;
        if (!((elements === null || elements === void 0 ? void 0 : elements.length) === 2 && elements[0].type === 'doctype' && hasSingleArticle(elements[1]))) {
            throw new Error('Element <article> is not the only element of the JATS');
        }
        this.doctype = elements[0].doctype;
        const converted = convertToUnist(elements[1]);
        this.tree = select('article', converted);
        (_a = this.log) === null || _a === void 0 ? void 0 : _a.debug(toc('Parsed and converted JATS to unist tree in %s'));
    }
    get frontmatter() {
        var _a, _b, _c, _d;
        const title = this.articleTitle;
        const subtitle = this.articleSubtitle;
        const date = this.publicationDate;
        const authors = this.articleAuthors;
        const firstSubject = select(Tags.subject, (_a = this.articleCategories) !== null && _a !== void 0 ? _a : this.front);
        const journalTitle = select(Tags.journalTitle, this.front);
        return {
            title: title ? toText(title) : undefined,
            subtitle: subtitle ? toText(subtitle) : undefined,
            doi: (_b = this.doi) !== null && _b !== void 0 ? _b : undefined,
            date: date ? (_c = toDate(date)) === null || _c === void 0 ? void 0 : _c.toISOString() : undefined,
            authors: authors === null || authors === void 0 ? void 0 : authors.map((a) => authorAndAffiliation(a, this.tree)),
            keywords: (_d = this.keywords) === null || _d === void 0 ? void 0 : _d.map((k) => toText(k)),
            venue: journalTitle ? { title: toText(journalTitle) } : undefined,
            subject: firstSubject ? toText(firstSubject) : undefined,
        };
    }
    get front() {
        return select(Tags.front, this.tree);
    }
    get premissions() {
        return select(Tags.permissions, this.front);
    }
    get doi() {
        var _a;
        return doi.normalize((_a = findArticleId(this.front, 'doi')) !== null && _a !== void 0 ? _a : '');
    }
    get pmc() {
        var _a;
        return (_a = findArticleId(this.front, 'pmc')) === null || _a === void 0 ? void 0 : _a.replace(/^PMC:?/, '');
    }
    get pmid() {
        return findArticleId(this.front, 'pmid');
    }
    get publicationDates() {
        return selectAll(Tags.pubDate, this.front);
    }
    get publicationDate() {
        return this.publicationDates.find((d) => !!select(Tags.day, d));
    }
    get license() {
        return select(Tags.license, this.premissions);
    }
    get keywordGroup() {
        return select(Tags.kwdGroup, this.front);
    }
    /** The first keywords */
    get keywords() {
        return selectAll(Tags.kwd, this.keywordGroup);
    }
    get keywordGroups() {
        return selectAll(Tags.kwdGroup, this.front);
    }
    get articleCategories() {
        return select(Tags.articleCategories, this.front);
    }
    get titleGroup() {
        return select(Tags.titleGroup, this.front);
    }
    get articleTitle() {
        return select(Tags.articleTitle, this.titleGroup);
    }
    get articleSubtitle() {
        return select(Tags.subtitle, this.titleGroup);
    }
    get abstract() {
        return select(Tags.abstract, this.front);
    }
    get abstracts() {
        return selectAll(Tags.abstract, this.front);
    }
    get contribGroup() {
        return select(Tags.contribGroup, this.front);
    }
    get contribGroups() {
        return selectAll(Tags.contribGroup, this.front);
    }
    get articleAuthors() {
        return selectAll(Tags.contrib, this.contribGroup);
    }
    get body() {
        return select(Tags.body, this.tree);
    }
    get back() {
        return select(Tags.back, this.tree);
    }
    get subArticles() {
        return selectAll(Tags.subArticle, this.tree);
    }
    get refList() {
        return select(Tags.refList, this.back);
    }
    get references() {
        return selectAll(Tags.ref, this.refList);
    }
}
function hasSingleArticle(element) {
    var _a;
    if (element.name === 'article') {
        return true;
    }
    if (element.name === 'pmc-articleset' &&
        ((_a = element.elements) === null || _a === void 0 ? void 0 : _a.length) === 1 &&
        element.elements[0].name === 'article') {
        return true;
    }
    return false;
}
